@@ -1,7 +1,7 @@
 package Data::Clean;
 
-our $DATE = '2016-05-21'; # DATE
-our $VERSION = '0.35'; # VERSION
+our $DATE = '2016-05-28'; # DATE
+our $VERSION = '0.36'; # VERSION
 
 use 5.010001;
 use strict;
@@ -65,9 +65,9 @@ sub command_unbless {
 
     # Data::Clone by default does not clone objects, so Acme::Damn can modify
     # the original object despite the use of clone(), so we need to know whether
-    # user runs clone_and_clean() or clean_in_place() and avoid the use of
-    # Acme::Damn for the former case. this workaround will be unnecessary when
-    # Data::Clone clones objects.
+    # user runs clone_and_clean() ($Data::Clean::_clone is true) or
+    # clean_in_place() and avoid the use of Acme::Damn for the former case. this
+    # workaround will be unnecessary when Data::Clone clones objects.
 
     my $acme_damn_available = eval { require Acme::Damn; 1 } ? 1:0;
     return join(
@@ -75,7 +75,8 @@ sub command_unbless {
         "if (!\$Data::Clean::_clone && $acme_damn_available) { ",
         "{{var}} = Acme::Damn::damn({{var}}) ",
         "} else { ",
-        "{{var}} = Function::Fallback::CoreOrPP::_unbless_fallback({{var}}) } ",
+        "{{var}} = Function::Fallback::CoreOrPP::_unbless_fallback({{var}}) ",
+        "} ",
         "\$ref = ref({{var}})",
     );
 }
@@ -97,8 +98,7 @@ sub command_clone {
         "if (++\$ctr_circ <= $limit) { ",
         "{{var}} = $clone_func({{var}}); redo ",
         "} else { ",
-        "{{var}} = 'CIRCULAR' } ",
-        "\$ref = ref({{var}})",
+        "{{var}} = 'CIRCULAR'; \$ref = '' }",
     );
 }
 
@@ -153,6 +153,17 @@ sub _generate_cleanser_code {
         $add_new_if->("\$ref eq '$ref'", $act0);
     };
 
+    # catch circular references
+    my $circ = $opts->{-circular};
+    if ($circ) {
+        my $meth = "command_$circ->[0]";
+        die "Can't handle command $circ->[0] for option '-circular'" unless $self->can($meth);
+        my @args = @$circ; shift @args;
+        my $act = $self->$meth(\@args);
+        #$add_stmt->('stmt', 'say "ref=$ref, " . {{var}}'); # DEBUG
+        $add_new_if->('$ref && $refs{ {{var}} }++', $act);
+    }
+
     # catch object of specified classes (e.g. DateTime, etc)
     for my $on (grep {/\A\w*(::\w+)*\z/} sort keys %$opts) {
         my $o = $opts->{$on};
@@ -174,17 +185,6 @@ sub _generate_cleanser_code {
         $add_if->($p->[1], $self->$meth(\@args));
     }
 
-    # catch circular references
-    my $circ = $opts->{-circular};
-    if ($circ) {
-        my $meth = "command_$circ->[0]";
-        die "Can't handle command $circ->[0] for option '-circular'" unless $self->can($meth);
-        my @args = @$circ; shift @args;
-        my $act = $self->$meth(\@args);
-        #$add_stmt->('stmt', 'say "ref=$ref, " . {{var}}'); # DEBUG
-        $add_if->('$ref && $refs{ {{var}} }++', $act);
-    }
-
     # recurse array and hash
     if ($opts->{'!recurse_obj'}) {
         $add_stmt->('stmt', 'my $reftype=Scalar::Util::reftype({{var}})//""');
@@ -272,7 +272,7 @@ Data::Clean - Clean data structure
 
 =head1 VERSION
 
-This document describes version 0.35 of Data::Clean (from Perl distribution Data-Clean), released on 2016-05-21.
+This document describes version 0.36 of Data::Clean (from Perl distribution Data-Clean), released on 2016-05-28.
 
 =head1 SYNOPSIS
 
